@@ -7,7 +7,7 @@ import types
 from collections import OrderedDict
 
 from django.db.models import Model
-from django.db.models.fields.files import FileField
+from django.db.models.fields.files import FieldFile
 from django.db.models.query import QuerySet
 from django.utils import formats, timezone
 from django.utils.encoding import force_text
@@ -22,10 +22,8 @@ from .converters import get_converter
 from .exception import NotAllowedException, UnsupportedMediaTypeException
 from .forms import RESTDictError, RESTDictIndexError, RESTListError
 from .utils import rfs
-from .utils.compatibility import (get_last_parent_pk_field_name,
-                                  get_reverse_field_name)
-from .utils.helpers import (ModelIteratorHelper, UniversalBytesIO,
-                            serialized_data_to_python, str_to_class)
+from .utils.compatibility import get_last_parent_pk_field_name, get_reverse_field_name
+from .utils.helpers import ModelIteratorHelper, UniversalBytesIO, serialized_data_to_python, str_to_class
 
 try:
     from django.db.models.fields.related import ForeignRelatedObjectsDescriptor, SingleRelatedObjectDescriptor
@@ -47,7 +45,20 @@ class Serializable:
         raise NotImplementedError
 
 
-class SerializableObj(Serializable):
+class SerializableObjMetaClass(type):
+
+    def __new__(cls, name, bases, attrs):
+        options = attrs.get('Meta')
+
+        if hasattr(options, 'fields'):
+            for field_name in options.fields:
+                attrs[field_name] = None
+
+        new_cls = type.__new__(cls, name, bases, attrs)
+        return new_cls
+
+
+class SerializableObj(Serializable, metaclass=SerializableObjMetaClass):
 
     resource_typemapper = {}
 
@@ -122,22 +133,21 @@ class RawVerboseValue:
     Return RAW, VERBOSE or BOTH values according to serialization type
     """
 
-    def __init__(self, raw_value, verbose_value, data_to_python_callback):
+    def __init__(self, raw_value, verbose_value):
         self.raw_value = raw_value
         self.verbose_value = verbose_value
-        self.data_to_python = data_to_python_callback
 
     def serialize(self, data, serialization_format, lazy=False, **kwargs):
         if serialization_format == Serializer.SERIALIZATION_TYPES.RAW:
-            return self.data_to_python(self.raw_value, serialization_format, lazy, **kwargs)
+            return self.raw_value
         elif serialization_format == Serializer.SERIALIZATION_TYPES.VERBOSE:
-            return self.data_to_python(self.verbose_value, serialization_format, lazy, **kwargs)
+            return self.verbose_value
         elif self.raw_value == self.verbose_value:
-            return self.data_to_python(self.raw_value, serialization_format, lazy, **kwargs)
+            return self.raw_value
         else:
             return {
-                '_raw': self.data_to_python(self.raw_value, serialization_format, lazy, **kwargs),
-                '_verbose': self.data_to_python(self.verbose_value, serialization_format, lazy, **kwargs),
+                '_raw': self.raw_value,
+                '_verbose': self.verbose_value,
             }
 
 
@@ -191,11 +201,13 @@ class Serializer:
 
     SERIALIZATION_TYPES = Enum('VERBOSE', 'RAW', 'BOTH')
 
-    def __init__(self, request=None):
+    def __init__(self, resource=None, request=None):
+        self.resource = resource
         self.request = request
 
     def _get_serializer(self, data):
-        return get_serializer(data, request=self.request)
+        resource_typemapper = self.resource.resource_typemapper if self.resource else None
+        return get_serializer(data, request=self.request, resource_typemapper=resource_typemapper)
 
     def _data_to_python(self, data, serialization_format, lazy=False, **kwargs):
         return self._get_serializer(data).serialize(data, serialization_format, **kwargs)
@@ -212,40 +224,32 @@ class Serializer:
         raise NotImplementedError
 
     def deserialize(self, data):
-        raise NotImplementedError
+        return data
 
 
 class ResourceSerializerMixin:
 
-    def __init__(self, resource, request=None):
-        self.resource = resource
-        super().__init__(request=request)
+    def _get_real_field_name(self, field_name):
+        return self.resource.renamed_fields.get(field_name, field_name)
 
-    def _get_resource(self):
-        return self.resource
-
-    def _get_serializer(self, data):
-        return get_serializer(data, request=self.request, resource_typemapper=self.resource.resource_typemapper)
-
-    def deserialize(self, data):
-        return data
-
-    def _serialize_recursive(self, data, serialization_format, requested_fieldset=None, **kwargs):
+    def serialize(self, data, serialization_format, requested_fieldset=None, **kwargs):
         if isinstance(data, dict):
-            return OrderedDict(
-                [
-                    (k, self._serialize_recursive(
-                        v,
-                        serialization_format,
-                        requested_fieldset=requested_fieldset.get(k).subfieldset if requested_fieldset else None,
-                        **kwargs)
-                     ) for k, v in data.items()
-                    if not requested_fieldset or k in requested_fieldset
-                ]
+            return LazyMappedSerializedData(
+                OrderedDict(
+                    [
+                        (k, self._serialize_other(
+                            v,
+                            serialization_format,
+                            requested_fieldset=requested_fieldset.get(k).subfieldset if requested_fieldset else None,
+                            **kwargs)
+                         ) for k, v in data.items()
+                        if not requested_fieldset or k in requested_fieldset
+                    ]
+                ), {v: k for k, v in self.resource.renamed_fields.items()}
             )
         elif isinstance(data, (list, tuple, set)):
             return (
-                self._serialize_recursive(v, serialization_format, requested_fieldset=requested_fieldset, **kwargs)
+                self._serialize_other(v, serialization_format, requested_fieldset=requested_fieldset, **kwargs)
                 for v in data
             )
         else:
@@ -256,21 +260,241 @@ class ResourceSerializerMixin:
         copy_kwargs['via'] = self.resource._get_via(copy_kwargs.get('via'))
         return self._data_to_python(data, serialization_format, **copy_kwargs)
 
-    def serialize(self, data, serialization_format, **kwargs):
-        return self._serialize_recursive(data, serialization_format, **kwargs)
-
 
 class ResourceSerializer(ResourceSerializerMixin, Serializer):
 
-    def serialize(self, data, serialization_format, **kwargs):
-        serialized_data = self._serialize_recursive(data, serialization_format, **kwargs)
-        if isinstance(serialized_data, dict):
-            return LazyMappedSerializedData(serialized_data, {v: k for k, v in self.resource.renamed_fields.items()})
+    pass
+
+
+class ObjectSerializer(Serializer):
+
+    obj_iterable_classes = (list, tuple)
+    obj_class = object
+
+    def _get_real_field_name(self, field_name):
+        return field_name
+
+    def _value_to_raw_verbose(self, val, obj, field_or_method=None, method_kwargs=None,
+                              serialization_format=None, **kwargs):
+        if hasattr(field_or_method, 'humanized') and field_or_method.humanized:
+            return RawVerboseValue(
+                self._data_to_python(val, serialization_format=Serializer.SERIALIZATION_TYPES.RAW, **kwargs),
+                self._data_to_python(
+                    field_or_method.humanized(val, obj, **(method_kwargs or {})),
+                    serialization_format=Serializer.SERIALIZATION_TYPES.RAW, **kwargs
+                )
+            )
         else:
-            return serialized_data
+            return val
+
+    def _obj_to_python(self, obj, serialization_format, requested_fieldset=None, extended_fieldset=None,
+                       exclude_fields=None, direct_serialization=False,
+                       serialized_objects=None, **kwargs):
+        exclude_fields = [] if exclude_fields is None else exclude_fields
+        serialized_objects = set() if serialized_objects is None else set(serialized_objects)
+        fieldset = self._get_fieldset(obj, extended_fieldset, requested_fieldset, exclude_fields,
+                                      kwargs.get('via'), direct_serialization, serialized_objects)
+        serialized_objects.add(self._get_obj_serialization_name(obj))
+        return self._fields_to_python(obj, serialization_format, fieldset, requested_fieldset,
+                                      serialized_objects=serialized_objects,
+                                      direct_serialization=direct_serialization, **self._get_subkwargs(kwargs))
+
+    def _field_to_python(self, field_name, real_field_name, obj, serialization_format, allow_tags=False, **kwargs):
+        obj_fields = set(obj.__class__.__dict__.keys())
+        if real_field_name == '_obj_name':
+            return self._data_to_python(
+                str(obj),
+                serialization_format,
+                allow_tags=allow_tags,
+                **kwargs
+            )
+        elif real_field_name in obj_fields:
+            return self._data_to_python(
+                self._value_to_raw_verbose(
+                    getattr(obj, real_field_name),
+                    obj,
+                    allow_tags=False,
+                    serialization_format=serialization_format,
+                    **kwargs
+                ),
+                serialization_format,
+                **kwargs
+            )
+        else:
+            raise SerializerFieldNotFound('Field "{}" was not found'.format(field_name))
+
+    def _fields_to_python(self, obj, serialization_format, fieldset, requested_fieldset, **kwargs):
+        python_data = {}
+        for field in fieldset.fields:
+            subkwargs = self._get_subkwargs(kwargs)
+            requested_field = None
+            if requested_fieldset:
+                requested_field = requested_fieldset.get(field.name)
+            field_name = self._get_field_name(field, requested_field, subkwargs)
+            real_field_name = self._get_real_field_name(field_name)
+            python_data[field_name] = self._field_to_python(
+                field_name, real_field_name, obj, serialization_format, **subkwargs
+            )
+        return python_data
+
+    def _get_subkwargs(self, kwargs):
+        subkwargs = kwargs.copy()
+        subkwargs['exclude_fields'] = None
+        subkwargs['via'] = kwargs.get('via')
+        return subkwargs
+
+    def _get_field_name(self, field, requested_field, subkwargs):
+        if field.subfieldset:
+            field_name, subkwargs['extended_fieldset'] = field.name, field.subfieldset
+        else:
+            field_name, subkwargs['extended_fieldset'] = field.name, None
+
+        if requested_field and requested_field.subfieldset:
+            subkwargs['requested_fieldset'] = requested_field.subfieldset
+        elif field.subfieldset:
+            subkwargs['requested_fieldset'] = field.subfieldset
+
+        return field_name
+
+    def _get_obj_serialization_name(self, obj):
+        return id(obj)
+
+    def _method_to_python(self, method, obj, serialization_format, allow_tags=False, requested_fieldset=None, **kwargs):
+        method_kwargs_names = inspect.getargspec(method)[0][1:]
+
+        method_kwargs = {}
+
+        fun_kwargs = {
+            'request': self.request,
+            'obj': obj,
+            'requested_fieldset': requested_fieldset
+        } if self.request else {
+            'obj': obj,
+            'requested_fieldset': requested_fieldset
+        }
+
+        for arg_name in method_kwargs_names:
+            if arg_name in fun_kwargs:
+                method_kwargs[arg_name] = fun_kwargs[arg_name]
+
+        if len(method_kwargs_names) == len(method_kwargs):
+            return self._data_to_python(
+                self._value_to_raw_verbose(
+                    method(**method_kwargs),
+                    obj,
+                    field_or_method=method,
+                    method_kwargs={k: v for k, v in method_kwargs.items() if k != 'obj'},
+                    serialization_format=serialization_format,
+                    allow_tags=allow_tags,
+                    **kwargs
+                ),
+                serialization_format, allow_tags=allow_tags or getattr(method, 'allow_tags', False),
+                requested_fieldset=requested_fieldset,
+                **kwargs
+            )
+        else:
+            raise SerializationException('Invalid method parameters')
+
+    def _get_fieldset(self, obj, extended_fieldset, requested_fieldset, exclude_fields, via, direct_serialization,
+                      serialized_objects):
+
+        if self._get_obj_serialization_name(obj) in serialized_objects:
+            return rfs((get_last_parent_pk_field_name(obj),))
+
+        direct_serialization_fields = self._get_direct_serialization_fields(obj)
+        allowed_fieldset = rfs(
+            requested_fieldset if requested_fieldset else (
+                direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields
+            )
+        )
+        default_fieldset = rfs(direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields)
+
+        if extended_fieldset:
+            default_fieldset.join(extended_fieldset)
+            allowed_fieldset.join(extended_fieldset)
+
+        if requested_fieldset:
+            # requested_fieldset must be cloned because RFS is not immutable and intersection change it
+            fieldset = rfs(requested_fieldset).intersection(allowed_fieldset)
+        else:
+            fieldset = default_fieldset.intersection(allowed_fieldset)
+
+        if exclude_fields:
+            fieldset.subtract(exclude_fields)
+        return fieldset
+
+    def serialize(self, data, serialization_format, **kwargs):
+        if isinstance(data, self.obj_iterable_classes):
+            return (self._obj_to_python(obj, serialization_format, **kwargs) for obj in data)
+        elif isinstance(data, self.obj_class):
+            return self._obj_to_python(data, serialization_format, **kwargs)
+        else:
+            raise RuntimeError('Invalid data type')
 
 
-class ObjectResourceSerializer(ResourceSerializerMixin, Serializer):
+class ObjectResourceSerializer(ResourceSerializerMixin, ObjectSerializer):
+
+    obj_iterable_classes = (list, tuple)
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.obj_class = self.resource.model
+
+    def _get_subkwargs(self, kwargs):
+        subkwargs = kwargs.copy()
+        subkwargs['exclude_fields'] = None
+        subkwargs['via'] = self.resource._get_via(kwargs.get('via'))
+        return subkwargs
+
+    def _get_fieldset_from_resource(self, obj, via, has_read_permission):
+        if not has_read_permission:
+            return self.resource.get_guest_fields_rfs(obj)
+        else:
+            return self.resource.get_general_fields_rfs(obj)
+
+    def _get_allowed_fieldset_from_resource(self, obj, via, has_read_permission):
+        if not has_read_permission:
+            return self.resource.get_guest_fields_rfs(obj)
+        else:
+            return self.resource.get_allowed_fields_rfs(obj)
+
+    def _field_to_python(self, field_name, real_field_name, obj, serialization_format, allow_tags=False, **kwargs):
+        resource_method_fields = (
+            self.resource.get_methods_returning_field_value([real_field_name]) if self.resource else {}
+        )
+
+        if real_field_name in resource_method_fields:
+            return self._method_to_python(
+                resource_method_fields[real_field_name], obj, serialization_format, allow_tags=allow_tags, **kwargs
+            )
+        else:
+            return super()._field_to_python(
+                field_name, real_field_name, obj, serialization_format, allow_tags=False, **kwargs
+            )
+
+    def _get_fieldset(self, obj, extended_fieldset, requested_fieldset, exclude_fields, via, direct_serialization,
+                      serialized_objects):
+
+        if self._get_obj_serialization_name(obj) in serialized_objects:
+            return rfs((get_last_parent_pk_field_name(obj),))
+
+        has_read_permission = self.resource.has_read_obj_permission(obj=obj, via=via)
+        default_fieldset = self._get_fieldset_from_resource(obj, via, has_read_permission)
+        allowed_fieldset = self._get_allowed_fieldset_from_resource(obj, via, has_read_permission)
+
+        if extended_fieldset:
+            default_fieldset.join(extended_fieldset)
+            allowed_fieldset.join(extended_fieldset)
+
+        if requested_fieldset:
+            # requested_fieldset must be cloned because RFS is not immutable and intersection change it
+            fieldset = rfs(requested_fieldset).intersection(allowed_fieldset)
+        else:
+            fieldset = default_fieldset.intersection(allowed_fieldset)
+
+        if exclude_fields:
+            fieldset.subtract(exclude_fields)
+        return fieldset
 
     def _serialize_recursive(self, data, serialization_format, **kwargs):
         if isinstance(data, self.resource.model):
@@ -280,6 +504,37 @@ class ObjectResourceSerializer(ResourceSerializerMixin, Serializer):
         else:
             return super()._serialize_recursive(data, serialization_format, **kwargs)
 
+    def serialize(self, data, serialization_format, requested_fieldset=None, **kwargs):
+        if isinstance(data, self.resource.model) and not kwargs.get('via') and not requested_fieldset:
+            requested_fieldset = self.resource.get_detailed_fields_rfs(obj=data)
+
+        if isinstance(data, self.obj_iterable_classes):
+            return (
+                self._obj_to_python(obj, serialization_format, **kwargs, requested_fieldset=requested_fieldset)
+                for obj in data
+            )
+        elif isinstance(data, self.obj_class):
+            return self._obj_to_python(data, serialization_format, **kwargs, requested_fieldset=requested_fieldset)
+        else:
+            return self._serialize_other(data, serialization_format, **kwargs)
+
+
+@register(FieldFile)
+class FileSerializer(Serializer):
+
+    def serialize(self, data, serialization_format, allow_tags=False, **kwargs):
+        if data:
+            filename = os.path.basename(data.name)
+            return {
+                'filename': filename,
+                'content_type': (
+                    mimetypes.types_map.get('.{}'.format(filename.split('.')[-1])) if '.' in filename else None
+                ),
+                'url': data.url,
+            }
+        else:
+            return None
+
 
 @register(str)
 class StringSerializer(Serializer):
@@ -295,13 +550,60 @@ class DefaultSerializer(Serializer):
         return force_text(data, strings_only=True)
 
 
-@register(datetime.datetime)
-class DateTimeSerializer(Serializer):
+class BaseRawVerboseValueSerializer(Serializer):
+
+    def _get_verbose_value(self, data, **kwargs):
+        raise NotImplementedError
+
+    def _get_raw_value(self, data):
+        return data
 
     def serialize(self, data, serialization_format, **kwargs):
+        value = self._get_raw_value(data)
+        if serialization_format == Serializer.SERIALIZATION_TYPES.RAW:
+            return value
+        else:
+            return self._data_to_python(
+                RawVerboseValue(
+                    value,
+                    self._get_verbose_value(value, **kwargs) if value else settings.NONE_HUMANIZED_VALUE
+                ),
+                serialization_format=serialization_format,
+                **kwargs
+            )
+
+
+@register(datetime.date)
+class DateSerializer(BaseRawVerboseValueSerializer):
+
+    def _get_verbose_value(self, data, **kwargs):
+        return formats.localize(data)
+
+
+@register(datetime.time)
+class TimeSerializer(BaseRawVerboseValueSerializer):
+
+    def _get_verbose_value(self, data, **kwargs):
+        return formats.localize(data)
+
+
+@register(datetime.datetime)
+class DateTimeSerializer(BaseRawVerboseValueSerializer):
+
+    def _get_verbose_value(self, data, **kwargs):
+        return formats.localize(timezone.template_localtime(data))
+
+    def _get_raw_value(self, data):
         return timezone.localtime(data) if timezone.is_aware(data) else data
 
 
+@register(bool)
+class BoolSerializer(BaseRawVerboseValueSerializer):
+
+    def _get_verbose_value(self, data, **kwargs):
+        return data and ugettext('Yes') or ugettext('No')
+
+
 @register(dict)
 class DictSerializer(Serializer):
 
@@ -347,14 +649,10 @@ class SerializableSerializer(Serializer):
 
 
 @register((Model, QuerySet, ModelIteratorHelper))
-class ModelSerializer(Serializer):
+class ModelSerializer(ObjectSerializer):
 
-    def _get_resource(self):
-        return None
-
-    def _get_real_field_name(self, field_name):
-        resource = self._get_resource()
-        return resource.renamed_fields.get(field_name, field_name) if resource else field_name
+    obj_class = Model
+    obj_iterable_classes = (ModelIteratorHelper, QuerySet)
 
     def _get_model_fields(self, obj):
         return {f.name: f for f in obj._meta.fields if hasattr(f, 'serialize') and f.serialize}
@@ -368,62 +666,36 @@ class ModelSerializer(Serializer):
             if (f.one_to_many or f.one_to_one) and f.auto_created and not f.concrete
         }
 
-    def _get_verbose_value(self, raw, field_or_method, obj, **kwargs):
-        if hasattr(field_or_method, 'humanized') and field_or_method.humanized:
-            return field_or_method.humanized(raw, obj, **kwargs)
-        elif hasattr(field_or_method, 'choices') and field_or_method.choices:
-            return getattr(obj, 'get_{}_display'.format(field_or_method.attname))()
-        if isinstance(raw, bool):
-            return raw and ugettext('Yes') or ugettext('No')
-        elif isinstance(raw, datetime.datetime):
-            return formats.localize(timezone.template_localtime(raw))
-        elif isinstance(raw, (datetime.date, datetime.time)):
-            return formats.localize(raw)
-        elif raw is None:
-            return settings.NONE_HUMANIZED_VALUE
-        else:
-            return raw
-
-    def _value_to_raw_verbose(self, val, field_or_method, obj, **kwargs):
-        return RawVerboseValue(
-            val,
-            self._get_verbose_value(val, field_or_method, obj, **kwargs),
-            data_to_python_callback=self._data_to_python
-        )
-
-    def _method_to_python(self, method, obj, serialization_format, allow_tags=False, requested_fieldset=None, **kwargs):
-        method_kwargs_names = inspect.getargspec(method)[0][1:]
-
-        method_kwargs = {}
-
-        fun_kwargs = {
-            'request': self.request,
-            'obj': obj,
-            'requested_fieldset': requested_fieldset
-        } if self.request else {
-            'obj': obj,
-            'requested_fieldset': requested_fieldset
-        }
-
-        for arg_name in method_kwargs_names:
-            if arg_name in fun_kwargs:
-                method_kwargs[arg_name] = fun_kwargs[arg_name]
-
-        if len(method_kwargs_names) == len(method_kwargs):
-            return self._data_to_python(
-                self._value_to_raw_verbose(
-                    method(**method_kwargs), method, obj, **{k: v for k, v in method_kwargs.items() if k != 'obj'}
+    def _value_to_raw_verbose(self, val,  obj, field_or_method=None, method_kwargs=None, serialization_format=None,
+                              **kwargs):
+        if hasattr(field_or_method, 'choices') and field_or_method.choices:
+            return RawVerboseValue(
+                self._data_to_python(val, serialization_format=Serializer.SERIALIZATION_TYPES.RAW, **kwargs),
+                self._data_to_python(
+                    getattr(obj, 'get_{}_display'.format(field_or_method.attname))(),
+                    serialization_format=Serializer.SERIALIZATION_TYPES.RAW,
+                    **kwargs
                 ),
-                serialization_format, allow_tags=allow_tags or getattr(method, 'allow_tags', False),
-                requested_fieldset=requested_fieldset,
-                **kwargs
             )
         else:
-            raise SerializationException('Invalid method parameters')
+            return super()._value_to_raw_verbose(
+                val, obj, field_or_method=field_or_method, method_kwargs=method_kwargs,
+                serialization_format=serialization_format, **kwargs
+            )
+
+    def _get_model_field_raw_value(self, obj, field):
+        return getattr(obj, field.attname)
 
     def _model_field_to_python(self, field, obj, serialization_format, allow_tags=False, **kwargs):
         return (self._lazy_data_to_python if field.is_relation else self._data_to_python)(
-            self._value_to_raw_verbose(self._get_model_field_raw_value(obj, field), field, obj)
+            self._value_to_raw_verbose(
+                self._get_model_field_raw_value(obj, field),
+                obj,
+                field_or_method=field,
+                serialization_format=serialization_format,
+                allow_tags=allow_tags,
+                **kwargs
+            )
             if not field.remote_field else getattr(obj, field.name),
             serialization_format, allow_tags=allow_tags or getattr(field, 'allow_tags', False), **kwargs
         )
@@ -450,52 +722,20 @@ class ModelSerializer(Serializer):
         kwargs['exclude_fields'] = self._get_reverse_excluded_fields(field, obj)
         return self._lazy_data_to_python(val, serialization_format, **kwargs)
 
-    def _copy_kwargs(self, resource, kwargs):
-        subkwargs = kwargs.copy()
-        subkwargs['exclude_fields'] = None
-        subkwargs['via'] = resource._get_via(kwargs.get('via')) if resource else kwargs.get('via')
-        return subkwargs
+    def _field_to_python(self, field_name, real_field_name, obj, serialization_format, allow_tags=False, **kwargs):
 
-    def _get_field_name(self, field, requested_field, subkwargs):
-        if field.subfieldset:
-            field_name, subkwargs['extended_fieldset'] = field.name, field.subfieldset
-        else:
-            field_name, subkwargs['extended_fieldset'] = field.name, None
-
-        if requested_field and requested_field.subfieldset:
-            subkwargs['requested_fieldset'] = requested_field.subfieldset
-        elif field.subfieldset:
-            subkwargs['requested_fieldset'] = field.subfieldset
-
-        return field_name
-
-    def _get_file_field_value(self, val):
-        if val:
-            filename = os.path.basename(val.name)
-            return {
-                'filename': filename,
-                'content_type': (
-                    mimetypes.types_map.get('.{}'.format(filename.split('.')[-1])) if '.' in filename else None
-                ),
-                'url': val.url,
-            }
-        else:
-            return None
-
-    def _get_model_field_raw_value(self, obj, field):
-        val = getattr(obj, field.attname)
-        return self._get_file_field_value(val) if isinstance(field, FileField) else val
-
-    def _field_to_python(self, field_name, resource_method_fields, model_fields, m2m_fields, reverse_fields,
-                         obj, serialization_format, allow_tags=False, **kwargs):
+        resource_method_fields = (
+            self.resource.get_methods_returning_field_value([real_field_name]) if self.resource else {}
+        )
+        model_fields = self._get_model_fields(obj)
+        m2m_fields = self._get_m2m_fields(obj)
+        reverse_fields = self._get_reverse_fields(obj)
 
         real_field_name = self._get_real_field_name(field_name)
         if real_field_name in resource_method_fields:
             return self._method_to_python(
                 resource_method_fields[real_field_name], obj, serialization_format, allow_tags=allow_tags, **kwargs
             )
-        elif real_field_name == '_obj_name':
-            return self._data_to_python(str(obj), serialization_format, allow_tags=allow_tags, **kwargs)
         elif real_field_name in m2m_fields:
             return self._m2m_field_to_python(
                 m2m_fields[real_field_name], obj, serialization_format, allow_tags=allow_tags, **kwargs
@@ -504,6 +744,13 @@ class ModelSerializer(Serializer):
             return self._model_field_to_python(
                 model_fields[real_field_name], obj, serialization_format, allow_tags=allow_tags, **kwargs
             )
+        elif real_field_name == '_obj_name':
+            return self._data_to_python(
+                str(obj),
+                serialization_format,
+                allow_tags=allow_tags,
+                **kwargs
+            )
         elif hasattr(obj.__class__, real_field_name):
             if real_field_name in reverse_fields:
                 val = getattr(obj, real_field_name, None) if hasattr(obj, real_field_name) else None
@@ -523,7 +770,14 @@ class ModelSerializer(Serializer):
             else:
                 method = get_class_method(obj, real_field_name)
                 return self._data_to_python(
-                    self._value_to_raw_verbose(val, method, obj),
+                    self._value_to_raw_verbose(
+                        val,
+                        obj,
+                        field_or_method=method,
+                        serialization_format=serialization_format,
+                        allow_tags=allow_tags,
+                        **kwargs
+                    ),
                     serialization_format,
                     allow_tags=allow_tags or method is not None and getattr(method, 'allow_tags', False),
                     **kwargs
@@ -531,40 +785,6 @@ class ModelSerializer(Serializer):
         else:
             raise SerializerFieldNotFound('Field "{}" was not found'.format(field_name))
 
-    def _fields_to_python(self, obj, serialization_format, fieldset, requested_fieldset, **kwargs):
-        resource = self._get_resource()
-        resource_method_fields = (
-            resource.get_methods_returning_field_value(fieldset.flat()) if resource else {}
-        )
-        model_fields = self._get_model_fields(obj)
-        m2m_fields = self._get_m2m_fields(obj)
-        reverse_fields = self._get_reverse_fields(obj)
-
-        python_data = OrderedDict()
-        for field in fieldset.fields:
-            subkwargs = self._copy_kwargs(resource, kwargs)
-            requested_field = None
-            if requested_fieldset:
-                requested_field = requested_fieldset.get(field.name)
-            field_name = self._get_field_name(field, requested_field, subkwargs)
-            python_data[field_name] = self._field_to_python(
-                field_name, resource_method_fields, model_fields, m2m_fields, reverse_fields,
-                obj, serialization_format, **subkwargs
-            )
-        return python_data
-
-    def _get_fieldset_from_resource(self, resource, obj, via, has_read_permission):
-        if not has_read_permission:
-            return resource.get_guest_fields_rfs(obj)
-        else:
-            return resource.get_general_fields_rfs(obj)
-
-    def _get_allowed_fieldset_from_resource(self, resource, obj, via, has_read_permission):
-        if not has_read_permission:
-            return resource.get_guest_fields_rfs(obj)
-        else:
-            return resource.get_allowed_fields_rfs(obj)
-
     def _get_direct_serialization_fields(self, obj):
         return rfs(obj._rest_meta.direct_serialization_fields).join(rfs(obj._rest_meta.default_fields))
 
@@ -574,19 +794,13 @@ class ModelSerializer(Serializer):
         if self._get_obj_serialization_name(obj) in serialized_objects:
             return rfs((get_last_parent_pk_field_name(obj),))
 
-        resource = self._get_resource()
-        if resource:
-            has_read_permission = resource.has_read_obj_permission(obj=obj, via=via)
-            default_fieldset = self._get_fieldset_from_resource(resource, obj, via, has_read_permission)
-            allowed_fieldset = self._get_allowed_fieldset_from_resource(resource, obj, via, has_read_permission)
-        else:
-            direct_serialization_fields = self._get_direct_serialization_fields(obj)
-            allowed_fieldset = rfs(
-                requested_fieldset if requested_fieldset else (
-                    direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields
-                )
+        direct_serialization_fields = self._get_direct_serialization_fields(obj)
+        allowed_fieldset = rfs(
+            requested_fieldset if requested_fieldset else (
+                direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields
             )
-            default_fieldset = rfs(direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields)
+        )
+        default_fieldset = rfs(direct_serialization_fields if direct_serialization else obj._rest_meta.guest_fields)
 
         if extended_fieldset:
             default_fieldset.join(extended_fieldset)
@@ -617,32 +831,10 @@ class ModelSerializer(Serializer):
                                       serialized_objects=serialized_objects,
                                       direct_serialization=direct_serialization, **kwargs)
 
-    def serialize(self, data, serialization_format, **kwargs):
-        if isinstance(data, ModelIteratorHelper):
-            return (self._obj_to_python(obj, serialization_format, **kwargs) for obj in data.iterator())
-        elif isinstance(data, QuerySet):
-            return (self._obj_to_python(obj, serialization_format, **kwargs) for obj in data)
-        elif isinstance(data, Model):
-            return self._obj_to_python(data, serialization_format, **kwargs)
-        else:
-            raise NotImplementedError
 
+class ModelResourceSerializer(ObjectResourceSerializer, ModelSerializer):
 
-class ModelResourceSerializer(ResourceSerializerMixin, ModelSerializer):
-
-    def _serialize_recursive(self, data, serialization_format, requested_fieldset=None, **kwargs):
-        if isinstance(data, self.resource.model) and not kwargs.get('via') and not requested_fieldset:
-            requested_fieldset = self.resource.get_detailed_fields_rfs(obj=data)
-
-        if (isinstance(data, self.resource.model) or
-              isinstance(data, (ModelIteratorHelper, QuerySet)) and issubclass(data.model, self.resource.model)):
-            return ModelSerializer.serialize(
-                self, data, serialization_format, requested_fieldset=requested_fieldset, **kwargs
-            )
-        else:
-            return super(ModelResourceSerializer, self)._serialize_recursive(
-                data, serialization_format, requested_fieldset=requested_fieldset, **kwargs
-            )
+    obj_iterable_classes = (ModelIteratorHelper, QuerySet)
 
 
 def serialize(data, requested_fieldset=None, serialization_format=Serializer.SERIALIZATION_TYPES.RAW,
